#include "doctab.h"
#include <QtWidgets>
#include <QtXml>


DocTab::DocTab(QWidget* p_parent) : QPlainTextEdit(p_parent),
  f_fDirectory(""),
  f_changed(false)
{

    connect(this, &DocTab::textChanged, this, &DocTab::sltDocChanged);

    lineNumberArea = new LineNumberArea(this);

    connect(this, &DocTab::blockCountChanged, this, &DocTab::updateLineNumberAreaWidth);
    connect(this, &DocTab::updateRequest, this, &DocTab::updateLineNumberArea);
    connect(this, &DocTab::cursorPositionChanged, this, &DocTab::highlightCurrentLine);

    updateLineNumberAreaWidth(0);

    lineNumberArea->hide();

}


QString DocTab::sltLoad()
{
    f_fDirectory = QFileDialog::getOpenFileName();

    if (f_fDirectory.isEmpty())
        return "";

    return sltLoad(f_fDirectory);
}

QString DocTab::sltLoad(const QString& p_fDirectory)
{
    f_fDirectory = p_fDirectory;
    QStringList list = p_fDirectory.split("/");
    f_fName = list.last();

    list.clear();

    list = f_fName.split(".");
    f_fExtension = list.last();



    QFile file(p_fDirectory);

    if (file.open(QIODevice::ReadOnly))
    {
        QTextStream stream(&file);

        disconnect(this, &DocTab::textChanged, this, &DocTab::sltDocChanged);
        setPlainText(stream.readAll());
        connect(this, &DocTab::textChanged, this, &DocTab::sltDocChanged);

        qDebug() << f_fName;
        emit changeWidTitle("", "");

        return f_fName;
    }
    return "";
}

void DocTab::sltIsCodeDoc()
{
    lineNumberArea->show();
    highlightCurrentLine();

    sltLoadHighlight();
}

void DocTab::sltSave()
{
    if (f_fDirectory.isEmpty())
    {
        sltSaveAs(f_fName);
        return;
    }

    QFile file(f_fDirectory);
    if(file.open(QIODevice::WriteOnly))
    {
        QTextStream(&file) << toPlainText();
        file.close();

        connect(this, &DocTab::textChanged, this, &DocTab::sltDocChanged);

        emit changeWidTitle("", "");
    }

    f_changed = false;
}

void DocTab::sltSaveAs(const QString& p_tip)
{
    QString newFileName = QFileDialog::getSaveFileName(0, f_fDirectory, p_tip);
    if (!newFileName.isEmpty())
    {
        QString lastFileName = f_fName;
        QString lastFileDirectory = f_fDirectory;

        f_fDirectory = newFileName;
        QStringList list = f_fDirectory.split("/");
        f_fName = list.last();

        list.clear();

        list = f_fName.split(".");
        f_fExtension = list.last();
        emit changeWidTitle(lastFileName, lastFileDirectory);
        sltSave();
    }
}

void DocTab::sltLoadHighlight()
{
    f_highlighter = new Highlighter(this->document(), f_fExtension);
    QString f_styleFName = ":/hLIni/highlightTest.xml";

    QFile styleFile(f_styleFName);
    QXmlInputSource src(&styleFile);
    QXmlSimpleReader reader;

    reader.setContentHandler(f_highlighter);

    qDebug() << "Ready to parse...";
    reader.parse(src);

}

void DocTab::sltDocChanged()
{
    qDebug() << "Doc " << f_fDirectory << " changed!";
    f_changed = true;
    disconnect(this, &DocTab::textChanged, this, &DocTab::sltDocChanged);
    emit fileChanded(f_fName);
}


void DocTab::sltSetChanged(bool p_was)
{
    f_changed = p_was;
}


int DocTab::lineNumberAreaWidth()
{
    int digits = 1;
    int max = qMax(1, blockCount());
    while (max >= 10) {
        max /= 10;
        ++digits;
    }

    int space = 3 + fontMetrics().horizontalAdvance(QLatin1Char('9')) * digits;

    return space;
}



void DocTab::updateLineNumberAreaWidth(int /* newBlockCount */)
{
    setViewportMargins(lineNumberAreaWidth(), 0, 0, 0);
}


void DocTab::updateLineNumberArea(const QRect &rect, int dy)
{
    if (dy)
        lineNumberArea->scroll(0, dy);
    else
        lineNumberArea->update(0, rect.y(), lineNumberArea->width(), rect.height());

    if (rect.contains(viewport()->rect()))
        updateLineNumberAreaWidth(0);
}

void DocTab::resizeEvent(QResizeEvent *e)
{
    QPlainTextEdit::resizeEvent(e);

    QRect cr = contentsRect();
    lineNumberArea->setGeometry(QRect(cr.left(), cr.top(), lineNumberAreaWidth(), cr.height()));
}

void DocTab::highlightCurrentLine()
{
    QList<QTextEdit::ExtraSelection> extraSelections;

    if (!isReadOnly())
    {
        QTextEdit::ExtraSelection selection;

        QColor lineColor = QColor(Qt::yellow).lighter(160);

        selection.format.setBackground(lineColor);
        selection.format.setProperty(QTextFormat::FullWidthSelection, true);
        selection.cursor = textCursor();
        selection.cursor.clearSelection();
        extraSelections.append(selection);
    }

    setExtraSelections(extraSelections);
}


void DocTab::lineNumberAreaPaintEvent(QPaintEvent *event)
{
    QPainter painter(lineNumberArea);
    painter.fillRect(event->rect(), Qt::lightGray);

    QTextBlock block = firstVisibleBlock();
    int blockNumber = block.blockNumber();
    int top = qRound(blockBoundingGeometry(block).translated(contentOffset()).top());
    int bottom = top + qRound(blockBoundingRect(block).height());


    while (block.isValid() && top <= event->rect().bottom())
    {
        if (block.isVisible() && bottom >= event->rect().top())
        {
            QString number = QString::number(blockNumber + 1);
            painter.setPen(Qt::black);
            painter.drawText(0, top, lineNumberArea->width(), fontMetrics().height(), Qt::AlignRight, number);
                }

                block = block.next();
                top = bottom;
                bottom = top + qRound(blockBoundingRect(block).height());
                ++blockNumber;
            }
        }
